/**
 * Labyrinthe carre
 */

public class Labyrinthe {
	private int taille;
	private boolean[][] mur;

	/**
	 * Construit une arene carree entouree d'un mur
	 * @param n taille du carre
	 */
	public Labyrinthe(int n) {
		if(n<0) n=0;		
		// Initialisation de l'attribut taille
    this.taille = n;
		// Initialisation de l'attribut mur : construction du tableau en memoire, puis
    // remplissage. Par defaut le tableau contient false dans chaque case,
    // il faut mettre les cases du bord a true.
		this.mur = new boolean[n][n];

		for(int i=0; i<n-1; i++) {
			this.mur[i][0] = true;
			this.mur[n-1][i] = true;
			this.mur[n-1-i][n-1] = true;
			this.mur[0][n-1-i] = true;
		}
	}

	/**
	 * Renvoie la taille du labyrinthe
	 * @return taille du labyrinthe
	 */
	public int getTaille() {
		return this.taille;
	}

	/**
	 * Remplit le labyrinthe de murs
	 */
	public void remplir() {
		for(int i=0; i<this.taille; i++)
			for(int j=0; j<this.taille; j++)
				this.mur[i][j] = true;
	}

  // La methode tronque prend un entier n et renvoie l'entier le plus proche
  // dans les bornes du tableau
	private int tronque(int n) {
    if(n<0) n=0;
		if(n>=this.taille) n=this.taille-1;
		return n;
	}
	/**
	 * Creuse un rectangle dans le labyrinthe
	 * @param x1 abscisse d'un coin A du rectangle
	 * @param y1 ordonnee du coin A
	 * @param x2 abscisse du coin B du rectangle oppose a A
	 * @param y2 ordonnee du coin B
	 */
	public void creuser(int x1, int y1, int x2, int y2) {
    // D'abord on remet les parametres dans le bon ordre
    // on veut x1<=x2 et y1<=y2
		if(x2<x1) {
			int tmp = x1;
			x1 = x2;
			x2 = tmp;
		}
		if(y2<y1) {
			int tmp = y1;
			y1 = y2;
			y2 = tmp;
		}
    // Ensuite on les fait "rentrer" dans le tableau
		x1=tronque(x1); x2=tronque(x2);
		y1=tronque(y1); y2=tronque(y2);
    // Maintenant on parcourt le rectangle
		for(int i=x1; i<=x2; i++)
			for(int j=y1; j<=y2; j++)
				this.mur[i][j] = false;
	}
	/**
	 * Test si une case ne contient pas de mur
	 * @param x abscisse de la case
	 * @param y ordonnee de la case
	 * @return true si la case ne contient pas de mur, false sinon
	 */
	public boolean etreVide(int x, int y) {
    // On est en dehors du tableau si tronquer x ou y renvoie une valeur differente
		if(x!=tronque(x) || y!=tronque(y)) return false;
    // Attention, this.mur[x][y] vaut true si la case
    // contient un mur et n'est donc PAS vide
		return !this.mur[x][y];
	}
	
}
