import java.util.Scanner;

/**
 * Personnage evoluant dans un Labyrinthe
 */

public class Personnage {
	private Labyrinthe labyrinthe;
	private int x, y;
	private int xSortie, ySortie;

	/**
	 * Construit un Personnage dans un labyrinthe donne
	 * @param lab Labyrinthe dans lequel evolue le Personnage. Si lab vaut null, le Labyrinthe est une arene de traille 10
	 */
	public Personnage(Labyrinthe lab) {
    // Initialisation de l'attribut labyrinthe
    if(lab==null)
			this.labyrinthe = new Labyrinthe(10);
		else
			this.labyrinthe = lab;

		int n = this.labyrinthe.getTaille();
		MesMaths mm = new MesMaths();

    // Initialisation des attributs x et y
    // On choisit une case au hasard, on recommence tant que la case contient un mur
		x = mm.choisirHasard(n); y = mm.choisirHasard(n);
		while(!this.labyrinthe.etreVide(x, y)) {
			x = mm.choisirHasard(n);
			y = mm.choisirHasard(n);
		}

    // Initialisation des attributs xSortie et ySortie
    // Meme principe
		xSortie = mm.choisirHasard(n); ySortie = mm.choisirHasard(n);
		while(!this.labyrinthe.etreVide(xSortie, ySortie)) {
			xSortie = mm.choisirHasard(n);
			ySortie = mm.choisirHasard(n);
		}
	}

	/**
	 * Renvoie une representation du labyrinthe sous forme de chaine de caracteres
	 * @return une chaine de caractere contenant :
	 * - des ' ' pour les cases vides
	 * - des 'X' pour les murs
	 * - un '.' pour le Personnage this
	 * - un 'S' pour la sortie
	 */
	public String toString() {
		String s = "";
		int n = this.labyrinthe.getTaille();
		char c;
    
		for(int j=0; j<n; j++) {
			for(int i=0; i<n; i++) {
        // On definit le caractere representant le contenu de la case
				if((i==x) && (j==y)) c='.';
				else if((i==xSortie) && (j==ySortie)) c='S';
				else if(this.labyrinthe.etreVide(i, j)) c=' ';
				else c='X';
        // On etend la chaine s avec ce caractere
				s=s+c;
			}
      // A la fin de la ligne on retourne... a la ligne
			s = s+'\n';
		}
		return s;
	}

	/**
	 * Fait se deplacer le Personnage this
	 * @param s chaine de caractere representant la direction du deplacement, "N", "S", "E" ou "O"
	 */
	public void seDeplacer(String s) {
		// On fait une copie (nx, ny) de la position (x, y)
    int nx = x, ny = y;
    // On deplace la position (nx, ny) selon la chaine s
		if(s.equals("N")) ny--;
		else if(s.equals("S")) ny++;
		else if(s.equals("O")) nx--;
		else if(s.equals("E")) nx++;
    // Si la nouvelle position est valide (libre et dans les limites),
    // on met a jour la position (x,y)
		if(this.labyrinthe.etreVide(nx, ny)) {
			x = nx;
			y = ny;
		}
	}

	/**
	 * Permet a l'utilisateur de faire se deplacer le Personnage jusqu'a la sortie
	 */
	public void jouer() {
		Scanner sc = new Scanner(System.in);
		// Tant que le Personnage this n'est pas arrive a la sortie, on joue
    // Attention la condition est un OU, pas un ET
    while((this.x != this.xSortie) || (this.y != ySortie)) {
			System.out.print(this);
			System.out.println("Deplacement ? ");
			String s = sc.nextLine();
			if(s.equals("X")) break;  // break fait sortir de la boucle while
			this.seDeplacer(s);
		}
	}

	// On ajoute une methode main. Ainsi on peut executer cette class (java Personnage),
	// ce qui appelle la methode main
	public static void main(String[] args) {
		Labyrinthe lab = new Labyrinthe(9);
		lab.remplir();
		lab.creuser(1,1,1,7);
		lab.creuser(1,1,7,1);
		lab.creuser(1,7,7,7);
		lab.creuser(3,3,7,3);
		lab.creuser(1,5,3,5);
		lab.creuser(3,4,3,4);
		lab.creuser(5,5,7,5);
    lab.creuser(7,6,7,6);
		Personnage p = new Personnage(lab);
		p.jouer();
	}
}
